@@ -7,7 +7,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.cpp 3115 2005-10-12 16:41:09Z mosu $
+   $Id: common.cpp 3195 2006-02-27 19:38:05Z mosu $
 
    helper functions, common variables
 
@@ -20,9 +20,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <iconv.h>
-#if defined(COMP_MSC)
-# include <libcharset.h>
-#elif HAVE_NL_LANGINFO
+#if HAVE_NL_LANGINFO
 # include <langinfo.h>
 #elif HAVE_LOCALE_CHARSET
 # include <libcharset.h>
@@ -740,6 +738,9 @@ _saferealloc(void *mem,
              size_t size,
              const char *file,
              int line) {
+  if (0 == size)
+    // Do this so realloc() may not return NULL on success.
+    size = 1;
   mem = realloc(mem, size);
   if (mem == NULL)
     die("common.cpp/saferealloc() called from file %s, line %d: realloc() "
@@ -852,6 +853,30 @@ strip(vector<string> &v,
     strip(v[i], newlines);
 }
 
+string &
+shrink_whitespace(string &s) {
+  int i;
+  bool previous_was_whitespace;
+
+  i = 0;
+  previous_was_whitespace = false;
+  while (s.length() > i) {
+    if (!isblanktab(s[i])) {
+      previous_was_whitespace = false;
+      ++i;
+      continue;
+    }
+    if (previous_was_whitespace)
+      s.erase(i, 1);
+    else {
+      previous_was_whitespace = true;
+      ++i;
+    }
+  }
+
+  return s;
+}
+
 string
 escape(const string &source) {
   string dst;
@@ -1588,27 +1613,49 @@ bool
 parse_timecode(const string &src,
                int64_t &timecode) {
   // Recognized format:
-  // 1. XXXXXXXs   with XXXXXX being a number of seconds followed
-  //    by the letter 's',
+  // 1. XXXXXXXu   with XXXXXX being a number followed
+  //    by one of the units 's', 'ms', 'us' or 'ns'
   // 2. HH:MM:SS.nnnnnnnnn  with up to nine digits 'n' for ns precision;
   // HH: is optional; HH, MM and SS can be either one or two digits.
+  // 2. HH:MM:SS:nnnnnnnnn  with up to nine digits 'n' for ns precision;
+  // HH: is optional; HH, MM and SS can be either one or two digits.
   int h, m, s, n, i, values[4], num_values, num_digits, num_colons;
   bool decimal_point_found;
 
   if (src.empty())
     return false;
 
-  // Number of seconds postfixed by 's'.
-  if ('s' == src[src.length() - 1]) {
-    string tmp = src;
+  try {
+    if (src.length() < 2)
+      throw false;
 
-    tmp.erase(tmp.length() - 1);
-    if (!parse_int(tmp, timecode))
-      return set_tcp_error("Non-digits found but expected a nunber of seconds "
-                           "before the 's' at the end");
+    string unit = src.substr(src.length() - 2, 2);
+
+    int64_t multiplier = 1000000000;
+    int unit_length = 2;
+    int64_t value = 0;
+
+    if (unit == "ms")
+      multiplier = 1000000;
+    else if (unit == "us")
+      multiplier = 1000;
+    else if (unit == "ns")
+      multiplier = 1;
+    else if (unit.substr(1, 1) == "s")
+      unit_length = 1;
+    else
+      throw false;
+
+    if (src.length() < (unit_length + 1))
+      throw false;
+
+    if (!parse_int(src.substr(0, src.length() - unit_length), value))
+      throw false;
+
+    timecode = value * multiplier;
 
-    timecode *= 1000000000ll;
     return true;
+  } catch (...) {
   }
 
   num_digits = 0;
@@ -1625,6 +1672,18 @@ parse_timecode(const string &src,
       values[num_values - 1] = values[num_values - 1] * 10 + src[i] - '0';
       ++num_digits;
 
+    } else if (('.' == src[i]) ||
+               ((':' == src[i]) && (2 == num_colons))) {
+      if (decimal_point_found)
+        return set_tcp_error("Invalid format: Second decimal point after "
+                             "first decimal point");
+
+      if (0 == num_digits)
+        return set_tcp_error("Invalid format: No digits before decimal point");
+      ++num_values;
+      num_digits = 0;
+      decimal_point_found = true;
+
     } else if (':' == src[i]) {
       if (decimal_point_found)
         return set_tcp_error("Invalid format: Colon inside nano-second part");
@@ -1636,17 +1695,6 @@ parse_timecode(const string &src,
       ++num_values;
       num_digits = 0;
 
-    } else if ('.' == src[i]) {
-      if (decimal_point_found)
-        return set_tcp_error("Invalid format: Second decimal point after "
-                             "first decimal point");
-
-      if (0 == num_digits)
-        return set_tcp_error("Invalid format: No digits before decimal point");
-      ++num_values;
-      num_digits = 0;
-      decimal_point_found = true;
-
     } else
       return set_tcp_error(mxsprintf("Invalid format: unknown character '%c' "
                                      "found", src[i]));
