@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: extern_data.h 3158 2005-12-05 16:31:01Z mosu $
+   $Id: extern_data.h 3213 2006-04-28 11:33:27Z mosu $
 
    declarations for ComboBox data
 
@@ -24,7 +24,7 @@ extern MTX_DLL_API const char *sub_charsets[];
 extern MTX_DLL_API const mime_type_t mime_types[];
 extern MTX_DLL_API const char *cctlds[];
 
-string MTX_DLL_API guess_mime_type(string ext);
+string MTX_DLL_API guess_mime_type(string ext, bool is_file);
 bool MTX_DLL_API is_valid_cctld(const string &s);
 
 #endif // __EXTERN_DATA_H
