@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_qtmp4.cpp 3147 2005-11-18 18:18:10Z mosu $
+   $Id: r_qtmp4.cpp 3206 2006-04-03 15:24:51Z mosu $
 
    Quicktime and MP4 reader
 
@@ -119,7 +119,8 @@ qtmp4_reader_c::~qtmp4_reader_c() {
 }
 
 qt_atom_t
-qtmp4_reader_c::read_atom(mm_io_c *read_from) {
+qtmp4_reader_c::read_atom(mm_io_c *read_from,
+                          bool exit_on_error) {
   qt_atom_t a;
 
   if (NULL == read_from)
@@ -134,8 +135,12 @@ qtmp4_reader_c::read_atom(mm_io_c *read_from) {
     a.hsize += 8;
   } else if (a.size == 0)
     a.size = file_size - read_from->getFilePointer() + 8;
-  if (a.size < a.hsize)
-    mxerror(PFX "Invalid chunk size " LLU " at " LLU ".\n", a.size, a.pos);
+  if (a.size < a.hsize) {
+    if (exit_on_error)
+      mxerror(PFX "Invalid chunk size " LLU " at " LLU ".\n", a.size, a.pos);
+    else
+      throw false;
+  }
 
   return a;
 }
@@ -309,7 +314,8 @@ qtmp4_reader_c::parse_headers() {
 
 void
 qtmp4_reader_c::update_tables(qtmp4_demuxer_ptr &dmx) {
-  uint32_t last, i, j, s, pts;
+  uint64_t last, s, pts;
+  int i, j;
 
   last = dmx->chunk_table.size();
 
@@ -392,6 +398,13 @@ qtmp4_reader_c::update_tables(qtmp4_demuxer_ptr &dmx) {
   mxverb(3, PFX "Frame offset table: %u entries\n",
          (unsigned int)dmx->frame_offset_table.size());
 
+  mxverb(4, PFX "Sample table contents: %u entries\n",
+         (unsigned int)dmx->sample_table.size());
+  for (i = 0; i < dmx->sample_table.size(); i++) {
+    qt_sample_t &sample = dmx->sample_table[i];
+    mxverb(4, PFX "  %d: pts " LLU " size %u pos " LLU "\n", i,
+           sample.pts, sample.size, sample.pos);
+  }
   update_editlist_table(dmx);
 }
 
@@ -437,7 +450,7 @@ qtmp4_reader_c::update_editlist_table(qtmp4_demuxer_ptr &dmx) {
     el.frames = sample - el.start_sample;
     frame += el.frames;
 
-    mxverb(4, "  %d: pts: %u  1st_sample: " LLU "  frames: %u (%5.3fs)  "
+    mxverb(4, "  %d: pts: " LLU "  1st_sample: " LLU "  frames: %u (%5.3fs)  "
            "pts_offset: " LLD "\n", i,
            el.pos, el.start_sample,  el.frames,
            (float)(el.duration) / (float)time_scale, el.pts_offset);
@@ -455,7 +468,12 @@ qtmp4_reader_c::parse_video_header_priv_atoms(qtmp4_demuxer_ptr &dmx,
     while (!mio.eof() && (mio.getFilePointer() < size)) {
       qt_atom_t atom;
 
-      atom = read_atom(&mio);
+      try {
+        atom = read_atom(&mio);
+      } catch (...) {
+        return;
+      }
+
       mxverb(2, PFX "%*sVideo private data size: %u, type: "
              "'%c%c%c%c'\n", (level + 1) * 2, "", (unsigned int)atom.size,
              BE2STR(atom.fourcc));
@@ -496,7 +514,11 @@ qtmp4_reader_c::parse_audio_header_priv_atoms(qtmp4_demuxer_ptr &dmx,
     while (!mio.eof() && (mio.getFilePointer() < (size - 8))) {
       qt_atom_t atom;
 
-      atom = read_atom(&mio);
+      try {
+        atom = read_atom(&mio, false);
+      } catch (...) {
+        return;
+      }
 
       if (FOURCC('e', 's', 'd', 's') != atom.fourcc) {
         mio.setFilePointer(atom.pos + 4);
@@ -675,18 +697,54 @@ void
 qtmp4_reader_c::handle_mdhd_atom(qtmp4_demuxer_ptr &new_dmx,
                                  qt_atom_t atom,
                                  int level) {
-  mdhd_atom_t mdhd;
+  int version;
 
-  if (atom.size < sizeof(mdhd_atom_t))
+  if (atom.size < 1)
     mxerror(PFX "'mdhd' atom is too small. Expected size: >= %u. Actual "
             "size: " LLD ".\n", (unsigned int)sizeof(mdhd_atom_t), atom.size);
-  if (io->read(&mdhd, sizeof(mdhd_atom_t)) != sizeof(mdhd_atom_t))
-    throw error_c("end-of-file");
-  mxverb(2, PFX "%*s Time scale: %u, duration: %u\n", level * 2, "",
-         get_uint32_be(&mdhd.time_scale),
-         get_uint32_be(&mdhd.duration));
-  new_dmx->time_scale = get_uint32_be(&mdhd.time_scale);
-  new_dmx->global_duration = get_uint32_be(&mdhd.duration);
+
+  version = io->read_uint8();
+
+  if (0 == version) {
+    mdhd_atom_t mdhd;
+
+    if (atom.size < sizeof(mdhd_atom_t))
+      mxerror(PFX "'mdhd' atom is too small. Expected size: >= %u. Actual "
+              "size: " LLD ".\n", (unsigned int)sizeof(mdhd_atom_t),
+              atom.size);
+    if (io->read(&mdhd.flags, sizeof(mdhd_atom_t) - 1) !=
+        (sizeof(mdhd_atom_t) - 1))
+      throw error_c("end-of-file");
+
+    mxverb(2, PFX "%*s Time scale: %u, duration: %u\n", level * 2, "",
+           get_uint32_be(&mdhd.time_scale),
+           get_uint32_be(&mdhd.duration));
+    new_dmx->time_scale = get_uint32_be(&mdhd.time_scale);
+    new_dmx->global_duration = get_uint32_be(&mdhd.duration);
+
+  } else if (1 == version) {
+    mdhd64_atom_t mdhd;
+
+    if (atom.size < sizeof(mdhd64_atom_t))
+      mxerror(PFX "'mdhd' atom is too small. Expected size: >= %u. Actual "
+              "size: " LLD ".\n", (unsigned int)sizeof(mdhd64_atom_t),
+              atom.size);
+    if (io->read(&mdhd.flags, sizeof(mdhd64_atom_t) - 1) !=
+        (sizeof(mdhd64_atom_t) - 1))
+      throw error_c("end-of-file");
+
+    mxverb(2, PFX "%*s Time scale: %u, duration: " LLU "\n", level * 2, "",
+           get_uint32_be(&mdhd.time_scale),
+           get_uint64_be(&mdhd.duration));
+    new_dmx->time_scale = get_uint32_be(&mdhd.time_scale);
+    new_dmx->global_duration = get_uint64_be(&mdhd.duration);
+
+  } else
+    mxerror(PFX "The 'media header' atom ('mdhd') uses the unsupported "
+            "version %d.\n", version);
+
+  if (0 == new_dmx->time_scale)
+    mxerror(PFX "The 'time scale' parameter was 0. This is not supported.\n");
 }
 
 void
@@ -779,7 +837,7 @@ qtmp4_reader_c::handle_mvhd_atom(qt_atom_t atom,
   if (io->read(&mvhd, sizeof(mvhd_atom_t)) != sizeof(mvhd_atom_t))
     throw error_c("end-of-file");
   time_scale = get_uint32_be(&mvhd.time_scale);
-  mxverb(2, PFX "%*s Time scale: %u\n", level * 2, "", time_scale);
+  mxverb(2, PFX "%*s Time scale: %un", level * 2, "", time_scale);
 }
 
 void
@@ -1119,8 +1177,8 @@ qtmp4_reader_c::handle_elst_atom(qtmp4_demuxer_ptr &new_dmx,
   mxverb(2, PFX "%*sEdit list table: %u entries\n", level * 2, "",
          count);
   for (i = 0; i < count; ++i)
-    mxverb(4, PFX "%*s%u: duration %u pos %u speed %u\n", (level + 1) * 2, "",
-           i, new_dmx->editlist_table[i].duration,
+    mxverb(4, PFX "%*s%u: duration " LLU " pos " LLU " speed %u\n",
+           (level + 1) * 2, "", i, new_dmx->editlist_table[i].duration,
            new_dmx->editlist_table[i].pos, new_dmx->editlist_table[i].speed);
 }
 
@@ -1179,12 +1237,11 @@ qtmp4_reader_c::handle_video_with_bframes(qtmp4_demuxer_ptr &dmx,
   is_bframe = false;
 
   if (dmx->pos == 0)
-    dmx->v_dts_offset = (int64_t)dmx->frame_offset_table[0] *
-      1000000000 / dmx->time_scale;
+    dmx->v_dts_offset = dmx->to_nsecs(dmx->frame_offset_table[0]);
 
   old_timecode = timecode;
-  timecode += (int64_t)dmx->frame_offset_table[dmx->pos] *
-    1000000000 / dmx->time_scale - dmx->v_dts_offset;
+  timecode += dmx->to_nsecs(dmx->frame_offset_table[dmx->pos]) -
+    dmx->v_dts_offset;
 
   PTZR(dmx->ptzr)->process(new packet_t(mem, timecode, duration, bref, fref));
 }
@@ -1210,9 +1267,9 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
         continue;
 
       io->setFilePointer(dmx->chunk_table[dmx->pos].pos);
-      timecode = 1000000000 *
-        ((uint64_t)dmx->chunk_table[dmx->pos].samples *
-         (uint64_t)dmx->duration) / (uint64_t)dmx->time_scale;
+      timecode =
+        dmx->to_nsecs((uint64_t)dmx->chunk_table[dmx->pos].samples *
+                      (uint64_t)dmx->duration);
 
       if (dmx->sample_size != 1) {
         if (!dmx->warning_printed) {
@@ -1258,10 +1315,9 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
       }
 
       if ((dmx->pos + 1) < dmx->chunk_table.size())
-        duration = 1000000000 *
-          ((uint64_t)dmx->chunk_table[dmx->pos + 1].samples *
-           (uint64_t)dmx->duration) /
-          (uint64_t)dmx->time_scale - timecode;
+        duration =
+          dmx->to_nsecs((uint64_t)dmx->chunk_table[dmx->pos + 1].samples *
+                        (uint64_t)dmx->duration);
       else
         duration = dmx->avg_duration;
       dmx->avg_duration = (dmx->avg_duration * dmx->pos + duration) /
@@ -1303,17 +1359,14 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
 
         // calc pts:
         timecode =
-          (dmx->sample_table[frame].pts +
-           dmx->editlist_table[dmx->editlist_pos].pts_offset) *
-          1000000000ll / dmx->time_scale;
+          dmx->to_nsecs(dmx->sample_table[frame].pts +
+                        dmx->editlist_table[dmx->editlist_pos].pts_offset);
 
       } else
-        timecode = (int64_t)dmx->sample_table[frame].pts * 1000000000 /
-          dmx->time_scale;
+        timecode = dmx->to_nsecs(dmx->sample_table[frame].pts);
 
       if ((frame + 1) < dmx->sample_table.size())
-        duration = (int64_t)dmx->sample_table[frame + 1].pts * 1000000000 /
-          dmx->time_scale - timecode;
+        duration = dmx->to_nsecs(dmx->sample_table[frame + 1].pts) - timecode;
       else
         duration = dmx->avg_duration;
       dmx->avg_duration = (dmx->avg_duration * frame + duration) /
@@ -1750,8 +1803,8 @@ qtmp4_demuxer_t::calculate_fps() {
     for (i = 0; i < sample_table.size(); ++i) {
       int64_t timecode;
 
-      timecode = ((int64_t)sample_table[i].pts +
-                  (int64_t)frame_offset_table[pos]) * 1000000000 / time_scale;
+      timecode = to_nsecs((int64_t)sample_table[i].pts +
+                          (int64_t)frame_offset_table[pos]);
       if (timecode > max_tc)
         max_tc = timecode;
     }
@@ -1783,3 +1836,17 @@ qtmp4_demuxer_t::calculate_fps() {
   return 0.0;
 }
 
+int64_t
+qtmp4_demuxer_t::to_nsecs(int64_t value) {
+  int i;
+
+  for (i = 1; i <= 100000000ll; i *= 10) {
+    int64_t factor = 1000000000ll / i;
+    int64_t value_up = value * factor;
+
+    if ((value_up / factor) == value)
+      return value_up / (time_scale / (1000000000ll / factor));
+  }
+
+  return value / (time_scale / 1000000000ll);
+}
