@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 3160 2005-12-06 16:17:01Z mosu $
+   \version $Id: mkvmerge.cpp 3213 2006-04-28 11:33:27Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -1237,17 +1237,8 @@ set_process_priority(int priority) {
 static string
 guess_mime_type_and_report(string file_name) {
   string mime_type;
-  int i;
 
-  i = file_name.rfind('.');
-  if (i < 0)
-    mxerror(_("No MIME type has been set for the attachment '%s', and "
-              "the file name contains no extension. Therefore the MIME "
-              "type could not be guessed automatically.\n"),
-            file_name.c_str());
-  file_name.erase(0, i + 1);
-
-  mime_type = guess_mime_type(file_name);
+  mime_type = guess_mime_type(file_name, true);
   if (mime_type != "") {
     mxinfo(_("Automatic MIME type recognition for '%s': %s\n"),
            file_name.c_str(), mime_type.c_str());
@@ -1255,7 +1246,7 @@ guess_mime_type_and_report(string file_name) {
   }
 
   mxerror(_("No MIME type has been set for the attachment '%s', and "
-            "it could not be guessed based on its extension.\n"),
+            "it could not be guessed.\n"),
           file_name.c_str());
   return "";
 }
