@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mkvmerge.h 2919 2005-05-21 18:09:39Z mosu $
+   $Id: mkvmerge.h 3176 2006-01-30 14:01:04Z mosu $
 
    definition of global variables found in mkvmerge.cpp
 
@@ -50,7 +50,8 @@ enum file_type_e {
   FILE_TYPE_WAVPACK4,
   FILE_TYPE_MPEG_PS,
   FILE_TYPE_USF,
-  FILE_TYPE_MAX = FILE_TYPE_USF
+  FILE_TYPE_COREPICTURE,
+  FILE_TYPE_MAX = FILE_TYPE_COREPICTURE
 };
 
 int64_t create_track_number(generic_reader_c *reader, int64_t tid);
