@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 3160 2005-12-06 16:17:01Z mosu $
+   $Id: mmg.cpp 3170 2005-12-14 15:30:23Z mosu $
 
    main stuff
 
@@ -1748,6 +1748,12 @@ mmg_dialog::set_output_maybe(const wxString &new_output) {
   }
 }
 
+void
+mmg_dialog::remove_output_filename() {
+  if (settings_page->cb_autoset_output_filename->IsChecked())
+    tc_output->SetValue(wxT(""));
+}
+
 void
 mmg_dialog::on_add_to_jobqueue(wxCommandEvent &evt) {
   wxString description, line;
